"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.querySortedTxsBetweenCycleRange = exports.queryProcessedTxsByCycleNumber = exports.queryProcessedTxByTxId = exports.bulkInsertProcessedTxs = exports.insertProcessedTx = void 0;
const db = require("./sqlite3storage");
const _1 = require("./");
const Logger = require("../Logger");
const Config_1 = require("../Config");
async function insertProcessedTx(processedTx) {
    try {
        const fields = Object.keys(processedTx).join(', ');
        const placeholders = Object.keys(processedTx).fill('?').join(', ');
        const values = db.extractValues(processedTx);
        const sql = 'INSERT INTO processedTxs (' +
            fields +
            ') VALUES (' +
            placeholders +
            ') ON CONFLICT (txId) DO UPDATE SET ' +
            'cycle = excluded.cycle, ' +
            'txTimestamp = excluded.txTimestamp, ' +
            'applyTimestamp = excluded.applyTimestamp';
        await db.run(_1.processedTxDatabase, sql, values);
        if (Config_1.config.VERBOSE) {
            Logger.mainLogger.debug('Successfully inserted ProcessedTransaction', processedTx.txId);
        }
    }
    catch (e) {
        Logger.mainLogger.error(e);
        Logger.mainLogger.error('Unable to insert ProcessedTransaction or it is already stored in to database', processedTx.txId);
    }
}
exports.insertProcessedTx = insertProcessedTx;
async function bulkInsertProcessedTxs(processedTxs) {
    try {
        const fields = Object.keys(processedTxs[0]).join(', ');
        const placeholders = Object.keys(processedTxs[0]).fill('?').join(', ');
        const values = db.extractValuesFromArray(processedTxs);
        let sql = 'INSERT INTO processedTxs (' + fields + ') VALUES (' + placeholders + ')';
        for (let i = 1; i < processedTxs.length; i++) {
            sql = sql + ', (' + placeholders + ')';
        }
        sql =
            sql +
                ' ON CONFLICT (txId) DO UPDATE SET ' +
                'cycle = excluded.cycle, ' +
                'txTimestamp = excluded.txTimestamp, ' +
                'applyTimestamp = excluded.applyTimestamp';
        await db.run(_1.processedTxDatabase, sql, values);
        if (Config_1.config.VERBOSE)
            Logger.mainLogger.debug('Successfully inserted ProcessedTransaction', processedTxs.length);
    }
    catch (e) {
        Logger.mainLogger.error(e);
        Logger.mainLogger.error('Unable to bulk insert ProcessedTransaction', processedTxs.length);
    }
}
exports.bulkInsertProcessedTxs = bulkInsertProcessedTxs;
async function queryProcessedTxByTxId(txId) {
    try {
        const sql = `SELECT * FROM processedTxs WHERE txId=?`;
        const processedTx = (await db.get(_1.processedTxDatabase, sql, [txId]));
        if (Config_1.config.VERBOSE) {
            Logger.mainLogger.debug('ProcessedTransaction txId', processedTx);
        }
        return processedTx;
    }
    catch (e) {
        Logger.mainLogger.error(e);
        return null;
    }
}
exports.queryProcessedTxByTxId = queryProcessedTxByTxId;
async function queryProcessedTxsByCycleNumber(cycleNumber) {
    try {
        const sql = `SELECT * FROM processedTxs WHERE cycle=?`;
        const processedTxs = (await db.all(_1.processedTxDatabase, sql, [cycleNumber]));
        if (Config_1.config.VERBOSE) {
            Logger.mainLogger.debug(`ProcessedTransactions for cycle: ${cycleNumber} ${processedTxs.length}`);
        }
        return processedTxs;
    }
    catch (e) {
        Logger.mainLogger.error(e);
        return null;
    }
}
exports.queryProcessedTxsByCycleNumber = queryProcessedTxsByCycleNumber;
async function querySortedTxsBetweenCycleRange(startCycle, endCycle) {
    try {
        const sql = `SELECT txId FROM processedTxs WHERE cycle BETWEEN ? AND ?`;
        const txIdsArray = (await db.all(_1.processedTxDatabase, sql, [startCycle, endCycle]));
        if (Config_1.config.VERBOSE) {
            Logger.mainLogger.debug(`txIds between ${startCycle} and ${endCycle} are ${txIdsArray ? txIdsArray.length : 0}`);
        }
        if (!txIdsArray) {
            return [];
        }
        const txIds = txIdsArray.map((tx) => tx.txId);
        txIds.sort();
        return txIds;
    }
    catch (e) {
        Logger.mainLogger.error('error in querySortedTxsBetweenCycleRange: ', e);
        return null;
    }
}
exports.querySortedTxsBetweenCycleRange = querySortedTxsBetweenCycleRange;
//# sourceMappingURL=processedTxs.js.map